(function() {
  var Log, async, ck, config, connectESI, express, fs, help, pantry, path, request, server, test;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  express = require('express');
  pantry = require('pantry');
  fs = require('fs');
  path = require('path');
  request = require('request');
  async = require('async');
  connectESI = require('connect-esi');
  ck = require('coffeekup');
  Log = require('coloured-log');
  config = {
    appName: "Timbits",
    engine: "coffee",
    port: 5678,
    home: process.cwd()
  };
  server = {};
  this.serve = function(options) {
    var key, value;
    for (key in options) {
      value = options[key];
      config[key] = value;
    }
    this.server = express.createServer();
    this.log = new Log(this.server.settings.env === 'development' ? Log.DEBUG : Log.INFO);
    this.server.register('.coffee', ck);
    this.server.set('views', "" + config.home + "/views");
    this.server.set('view engine', config.engine);
    this.server.set('view options', {
      layout: false
    });
    this.server.configure(__bind(function() {
      this.server.use(connectESI.setupESI());
      this.server.use(express.static("" + config.home + "/public"));
      this.server.use(express.static("" + config.home + "/views"));
      this.server.use(express.bodyParser());
      return this.server.use(express.cookieParser());
    }, this));
    this.server.configure('development', __bind(function() {
      return this.server.use(express.errorHandler({
        dumpExceptions: true,
        showStack: true
      }));
    }, this));
    this.server.configure('production', __bind(function() {
      return this.server.use(express.errorHandler());
    }, this));
    this.server.get('/', function(req, res) {
      return res.redirect('/timbits/help');
    });
    this.server.get('/timbits/help', __bind(function(req, res) {
      return res.send(ck.render(help, {
        context: this.box
      }));
    }, this));
    this.server.get('/timbits/test', __bind(function(req, res) {
      return res.send(ck.render(test, {
        context: null
      }));
    }, this));
    path = "" + config.home + "/timbits";
    fs.readdir(path, __bind(function(err, files) {
      var file, name, _i, _len, _results;
      if (err) {
        throw err;
      }
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        name = file.replace('.coffee', '');
        _results.push(this.add(name, require("" + path + "/" + name)));
      }
      return _results;
    }, this));
    this.server.listen(process.env.PORT || process.env.C9_PORT || config.port);
    this.log.info("Timbits server listening on port " + (this.server.address().port) + " in " + this.server.settings.env + " mode");
    server.address = this.server.address().address;
    server.port = this.server.address().port;
    return this.server;
  };
  this.box = {};
  this.add = function(name, timbit) {
    this.log.notice("Placing " + name + " in the box");
    timbit.name = name;
    this.box[name] = timbit;
    this.server.get("/" + name + "/help", function(req, res) {
      return res.send(ck.render(timbit.help, {
        context: timbit
      }));
    });
    this.server.get("/" + name + "/test", function(req, res) {
      return timbit.test(server, timbit, function(results) {
        results.timbit = timbit.name;
        return res.send(ck.render(test, {
          context: results
        }));
      });
    });
    return this.server.get("/" + name + "/:view?", function(req, res) {
      var attr, context, k, key, v, value, _base, _ref, _ref2, _ref3, _ref4;
      try {
        context = {};
        _ref = req.query;
        for (k in _ref) {
          v = _ref[k];
          context[k] = v;
        }
        context.name = name;
        context.view = (_ref2 = (_base = req.params).view) != null ? _ref2 : _base.view = 'default';
        _ref3 = timbit.params;
        for (key in _ref3) {
          attr = _ref3[key];
          if ((_ref4 = context[key]) == null) {
            context[key] = attr["default"];
          }
          value = context[key];
          if (attr.required && !value) {
            throw "" + key + " is a required parameter";
          }
          if (value && attr.strict && attr.values.indexOf(value) === -1) {
            throw "" + value + " is not a valid value for " + key + ".  Must be one of [" + (attr.values.join()) + "]";
          }
          if (value instanceof Array && !attr.multiple) {
            throw "" + key + " must be a single value";
          }
        }
        return timbit.eat(req, res, context);
      } catch (ex) {
        log.error("" + req.url + " - " + ex);
        throw ex;
      }
    });
  };
  help = function() {
    style('body {\n	background: #d2d5dc; /* Old browsers */\n	background: -moz-linear-gradient(top, #d2d5dc 0%, #ffffff 75%); /* FF3.6+ */\n	background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#d2d5dc), color-stop(75%,#ffffff)); /* Chrome,Safari4+ */\n	background: -webkit-linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* Chrome10+,Safari5.1+ */\n	background: -o-linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* Opera11.10+ */\n	background: -ms-linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* IE10+ */\n	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr=\'#d2d5dc\', endColorstr=\'#ffffff\',GradientType=0 ); /* IE6-9 */\n	background: linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* W3C */\n	background-repeat: no-repeat;\n	margin: 0;\n	padding: 0;\n	font: 14px Tahoma, Geneva, sans-serif;\n}\n#content {\n    background: #fff none;\n    width: 750px;\n    padding: 5px 20px 20px 20px;\n	box-shadow: 0 0px 20px #666;\n	min-height: 500px;\n}\n\na {color: #4E5989; text-decoration: underline;}\na:hover {text-decoration: none;}\n\n#wrapper {width: 750px; margin: 0 auto;}\n\nh1 {border-bottom: 1px solid #999; width: 100%; font: 30px Tahoma, Geneva, sans-serif}\n\np, h3 {margin: 0 0 5px 0;}\nh2 {margin: 40px 0 5px 0;}\n\nul {\n    margin: 0 0 10px 0;\n    list-style: none;\n	padding: 0;\n}\n\nli {padding: 0 0 10px 0; text-transform: uppercase;}');
    return div({
      id: 'wrapper'
    }, function() {
      return div({
        id: 'content'
      }, function() {
        h1('Timbits - Help');
        return ul(function() {
          var k, v, _results;
          _results = [];
          for (k in this) {
            v = this[k];
            _results.push(li(function() {
              return a({
                href: "/" + k + "/help"
              }, function() {
                return k + ' &raquo;';
              });
            }));
          }
          return _results;
        });
      });
    });
  };
  test = function() {
    var k, v;
    h1(function() {
      return "Timbits - " + this.timbit;
    });
    if (this.views) {
      ul(function() {
        var view, _i, _len, _ref, _results;
        _ref = this.views;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          view = _ref[_i];
          _results.push(li(function() {
            return view;
          }));
        }
        return _results;
      });
    }
    if (this.failed) {
      p(function() {
        return "Errors in testing " + this.failed;
      });
    }
    if (this.response) {
      this.response;
    }
    for (k in this) {
      v = this[k];
      p(function() {
        return "" + k + " [" + v + "]";
      });
    }
    if (this.failures) {
      return ul(function() {
        var failure, _i, _len, _ref, _results;
        _ref = this.failures;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          failure = _ref[_i];
          _results.push(li(function() {
            return "" + failure.timbit + " : " + failure.error;
          }));
        }
        return _results;
      });
    }
  };
  this.Timbit = (function() {
    function Timbit() {}
    Timbit.prototype.render = function(req, res, context) {
      var output;
      if (context.remote === 'true') {
        output = "$().ready(function() {\n	return $.get(\"/" + context.name + "/" + context.view + ".coffee\", function(data) {\n		context = " + (JSON.stringify(context)) + ";\n		return $('#" + context.timbit_id + "').html(CoffeeKup.render(data, context));\n	});\n});";
        res.setHeader("Content-Type", "text/plain");
        res.write(output);
        return res.end();
      } else {
        return res.render("" + context.name + "/" + context.view, {
          context: context
        });
      }
    };
    Timbit.prototype.fetch = function(req, res, context, key, options, callback) {
      if (callback == null) {
        callback = this.render;
      }
      return pantry.fetch(options, __bind(function(error, results) {
        context[key] = results;
        return callback(req, res, context);
      }, this));
    };
    Timbit.prototype.eat = function(req, res, context) {
      return this.render(req, res, context);
    };
    Timbit.prototype.help = function() {
      style('body {\n		background: #d2d5dc; /* Old browsers */\n		background: -moz-linear-gradient(top, #d2d5dc 0%, #ffffff 75%); /* FF3.6+ */\n		background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#d2d5dc), color-stop(75%,#ffffff)); /* Chrome,Safari4+ */\n		background: -webkit-linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* Chrome10+,Safari5.1+ */\n		background: -o-linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* Opera11.10+ */\n		background: -ms-linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* IE10+ */\n		filter: progid:DXImageTransform.Microsoft.gradient( startColorstr=\'#d2d5dc\', endColorstr=\'#ffffff\',GradientType=0 ); /* IE6-9 */\n		background: linear-gradient(top, #d2d5dc 0%,#ffffff 75%); /* W3C */\n		background-repeat: no-repeat;\n		margin: 0;\n		padding: 0;\n		font: 14px Tahoma, Geneva, sans-serif;\n	}\n	#content {\n	    background: #fff none;\n	    width: 750px;\n	    padding: 5px 20px 20px 20px;\n		box-shadow: 0 0px 20px #666;\n		min-height: 500px;\n	}\n\n	a {color: #4E5989; text-decoration: underline;}\n	a:hover {text-decoration: none;}\n\n	#wrapper {width: 750px; margin: 0 auto;}\n\n	h1 {border-bottom: 1px solid #999; width: 100%; font: 30px Tahoma, Geneva, sans-serif}\n	h2 {font: 20px Tahoma, Geneva, sans-serif}\n\n	p, h3 {margin: 0 0 5px 0;}\n	h2 {margin: 40px 0 5px 0;}\n\n	ul {\n	    margin: 0 0 10px 0;\n	    list-style: none;\n		padding: 0;\n	}\n\n	li {padding: 0 0 10px 0; text-transform: uppercase;}\n	#return {margin-top: 20px;}\n	table {border: 1px solid #4E5989; width: 100%; border-collapse: collapse; border-spacing: 0;}\n	th, tr {text-align: left; border-bottom: 1px solid #4E5989;}\n	td, th {border-right: 1px solid #4E5989;}\n	td, th {padding: 5px;}');
      return div({
        id: 'wrapper'
      }, function() {
        return div({
          id: 'content'
        }, function() {
          h1(this.name);
          p(this.about || 'Developer was too lazy to describe this widget');
          h2('Examples:');
          if (this.examples) {
            ul(function() {
              var example, _i, _len, _ref, _results;
              _ref = this.examples;
              _results = [];
              for (_i = 0, _len = _ref.length; _i < _len; _i++) {
                example = _ref[_i];
                _results.push(li(function() {
                  return a({
                    href: example.href
                  }, function() {
                    return example.caption;
                  });
                }));
              }
              return _results;
            });
          } else {
            p('Developer was too lazy to define any examples.');
          }
          h2('Parameters');
          if (this.params) {
            table(function() {
              return tbody(function() {
                var attr, key, _ref, _results;
                tr(function() {
                  th('Name');
                  th('Description');
                  th('Type');
                  th('Required');
                  th('Multiple');
                  th('Default');
                  return th('Values');
                });
                _ref = this.params;
                _results = [];
                for (key in _ref) {
                  attr = _ref[key];
                  _results.push(tr(function() {
                    td(key);
                    td(attr.description);
                    td(attr.type || 'String');
                    td((attr.required || false).toString());
                    td((attr.multiple || false).toString());
                    td(attr["default"]);
                    if (attr.values) {
                      return td(function() {
                        if (attr.strict) {
                          text('One of:');
                        } else {
                          text('Examples:');
                        }
                        return ul(function() {
                          var value, _i, _len, _ref2, _results2;
                          _ref2 = attr.values;
                          _results2 = [];
                          for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
                            value = _ref2[_i];
                            _results2.push(li(value));
                          }
                          return _results2;
                        });
                      });
                    }
                  }));
                }
                return _results;
              });
            });
          } else {
            p('None defined');
          }
          return div({
            id: 'return'
          }, function() {
            return a({
              href: '/timbits/help'
            }, function() {
              return '&laquo; Help Index';
            });
          });
        });
      });
    };
    Timbit.prototype.test = function(server, context, callback) {
      var results;
      results = {
        timbit: '',
        views: [],
        required: [],
        optional: [],
        failures: [],
        queries: [],
        failed: false
      };
      return async.series({
        testViews: function(callback) {
          return fs.readdir("" + config.home + "/views/" + context.name, function(err, list) {
            var failure, file, pending, _i, _len, _results;
            if (err || list === void 0) {
              failure = {
                timbit: "" + context.name,
                view: "",
                statusCode: "",
                querystring: "",
                error: "Unable to retrieve a list of views"
              };
              results.failures.push(failure);
              results.views.push('default');
              return callback(null);
            } else {
              pending = list.length;
              _results = [];
              for (_i = 0, _len = list.length; _i < _len; _i++) {
                file = list[_i];
                _results.push((function(file) {
                  if (file.match(/\.coffee/) != null) {
                    results.views.push(require('path').basename(file, '.coffee'));
                    if (!--pending) {
                      return callback(null);
                    }
                  }
                })(file));
              }
              return _results;
            }
          });
        },
        testParams: function(callback) {
          var k, param, v, _ref;
          _ref = context.params;
          for (k in _ref) {
            v = _ref[k];
            param = "" + k + "=" + v.values[0];
            if (v.required) {
              results.required.push(param);
            } else {
              results.optional.push(param);
            }
          }
          return callback(null);
        },
        testQueries: function(callback) {
          var _ref, _ref2, _ref3;
          if ((_ref = results.required) != null ? _ref.length : void 0) {
            if ((_ref2 = results.optional) != null ? _ref2.length : void 0) {
              results.queries.push("" + (results.required.join('&')) + "&" + (results.optional.join('&')));
            }
            results.queries.push(results.required.join('&'));
          } else {
            if ((_ref3 = results.optional) != null ? _ref3.length : void 0) {
              results.queries.push("" + (results.optional.join('&')));
            }
          }
          return callback(null);
        },
        testRunQueries: function(callback) {
          var pending, query, view, _i, _j, _len, _len2, _ref, _ref2, _ref3, _results, _results2;
          console.log("\n\n\tTesting Timbit - " + context.name + "\n\t-------------------------------------------------\n");
          if (!((_ref = results.queries) != null ? _ref.length : void 0)) {
            pending = results.views.length;
            _ref2 = results.views;
            _results = [];
            for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
              view = _ref2[_i];
              _results.push((function(view) {
                console.log("\tTesting view: " + view + " with \n\t\t uri: http://" + server.address + ":" + server.port + "/" + context.name + "/" + view);
                return request({
                  uri: "http://" + server.address + ":" + server.port + "/" + context.name + "/" + view
                }, function(error, response, body) {
                  var failure;
                  if (error) {
                    failure = {
                      timbit: "" + context.name,
                      view: "" + view,
                      statusCode: "" + response.statusCode,
                      querystring: "",
                      error: "" + error
                    };
                    console.log("\n** \t\t Errors: " + (error || body));
                    results.failures.push(failure);
                    results.failed = true;
                  }
                  if (!--pending) {
                    return callback(null);
                  }
                });
              })(view));
            }
            return _results;
          } else {
            pending = results.queries.length;
            _ref3 = results.queries;
            _results2 = [];
            for (_j = 0, _len2 = _ref3.length; _j < _len2; _j++) {
              query = _ref3[_j];
              _results2.push((function(query) {
                var view, _k, _len3, _ref4, _results3;
                _ref4 = results.views;
                _results3 = [];
                for (_k = 0, _len3 = _ref4.length; _k < _len3; _k++) {
                  view = _ref4[_k];
                  _results3.push((function(view) {
                    console.log("\tTesting view: " + view + " with query " + query + " \n\t\t uri: http://" + server.address + ":" + server.port + "/" + context.name + "/" + view + "?" + query);
                    return request({
                      uri: "http://" + server.address + ":" + server.port + "/" + context.name + "/" + view + "?" + query
                    }, function(error, response, body) {
                      var failure;
                      if ((response != null ? response.statusCode : void 0) >= 400 || error) {
                        failure = {
                          timbit: "" + context.name,
                          view: "" + view,
                          statusCode: "" + response.statusCode,
                          querystring: "" + query,
                          error: "" + (error || body)
                        };
                        console.log("\n** \t\t Errors: " + (error || body));
                        results.failures.push(failure);
                        results.failed = true;
                      }
                      if (!--pending) {
                        return callback(null);
                      }
                    });
                  })(view));
                }
                return _results3;
              })(query));
            }
            return _results2;
          }
        },
        testRunExamples: function(callback) {
          var example, pending, _i, _len, _ref, _results;
          if (context.examples != null) {
            pending = context.examples.length;
            _ref = context.examples;
            _results = [];
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
              example = _ref[_i];
              _results.push((function(example) {
                console.log("\tTesting example: " + example.caption + " with \n\t\t uri: http://" + server.address + ":" + server.port + example.href);
                return request({
                  uri: "http://" + server.address + ":" + server.port + example.href
                }, function(error, response, body) {
                  var failure;
                  if ((response != null ? response.statusCode : void 0) >= 400 || error) {
                    failure = {
                      timbit: "" + context.name,
                      view: "",
                      statusCode: "" + response.statusCode,
                      querystring: "",
                      error: "Using example " + example.href + " - " + (error || body)
                    };
                    console.log("\n** \t\t Errors: " + (error || body));
                    results.failures.push(failure);
                    results.failed = true;
                  }
                  if (!--pending) {
                    return callback(null);
                  }
                });
              })(example));
            }
            return _results;
          } else {
            return callback(null);
          }
        }
      }, function(error) {
        if (error) {
          console.log("Errors: " + error);
        }
        return callback(results);
      });
    };
    return Timbit;
  })();
}).call(this);
